/*
 * ----------------------------------------------------------------------------
 *
 *   Hatena::Antenna Plugin for Google Desktop Sidebar.
 *
 *  Copyright (c) 2006, Tennetiss
 *  http://www.akaiho.com/
 *
 *
 *  CHANGELOG:
 *    2006-02-16  :  initially setup
 *    2006-02-18  :  got options to show date and set time interval
 *    2006-02-24  :  got option to omit showing the first day
 *    2006-04-13  :  shows the updated time of each uri
 *                   handles the GD's alert window to show updates of each item
 *                   shows brief description of each update
 *    2006-04-22  :  got option to swtich whther showing detail of updates or not
 *    2006-04-23  :  bug fixed. (an empty description tag does not generate an instance)
 *
 *  ----------------------------------------------------------------------------
 */

var kCurrentVersion = "1.45";
var kUpdatesUri = "http://www.akaiho.com/projects/hatenabookmarkplugin";
var kProductHomePageUri = "http://www.akaiho.com/projects/hatenabookmarkplugin";

var kHatenaUsernameTemplate = "http://a.hatena.ne.jp/{user}/rss";
//var kHatenaUsernameDef = "http://b.hatena.ne.jp/{user}/rss";
var kDefaultUsername = "sample";
//var kCheckIntervalMs = 1000 * 3600 * 1; // refresh hourly.
var kCheckIntervalMs;
var kMaxAntennas = 500;
var kDefaultScoreForNewItem = 10000; // so new items appear at the top.
var kMaxScoreLimit = 1000000; // 
var kShowsDate;
var kShowsFirstDate;
var kLastUpdate;
var kDetail;

// Represents indexes inside a comma-separated value stored in options:
var kPropertyCount = 2; // how many props per url?
var kMetaDataIndex_Score = 0;
var kMetaDataIndex_Pinned = 1;

var timerId = null;

var http = null;
var contentItems = new Array();


Main();


function Main() {
  SetDefaultOptions();

  antenna.contentFlags = gddContentFlagNone | gddContentFlagHaveDetails;
  antenna.maxContentItems = kMaxAntennas;

  pluginHelper.onAddCustomMenuItems = AddCustomMenuItems;
  pluginHelper.onShowOptionsDlg = ShowOptionsDlg;

  kShowsDate = options.GetValue("Showsdate");
  kShowsFirstDate = options.GetValue("Showsfirstdate");
  ResetTimeInterval(options.GetValue("Timeinterval"));
  kDetail = options.GetValue("Showsdetail");
  var tmpDate = new Date(options.GetValue( "Lastupdate" ));
  kLastUpdate = tmpDate.getTime();
//  timerId = utils.setInterval(RefreshDisplay, kCheckIntervalMs);
  RefreshDisplay();
}


function view_onopen(){
  Main();
}

function ResetTimeInterval(newIntervalmin){
  kCheckIntervalMs = 1000 * 60 * newIntervalmin;
  if(timerId != null) { view.clearInterval(timerId); }
  timerId = view.setInterval(RefreshDisplay, kCheckIntervalMs);
}

function SetDefaultOptions() {
  options.defaultValue("Username") = kDefaultUsername;
  options.defaultValue("Timeinterval") = "60";
  options.defaultValue("Showsdate") = "1";
  options.defaultValue("Showsfirstdate") = "1";
  options.defaultValue("Lastupdate") = "0";
  options.defaultValue("Showsdetail") = "1";
}

function AddCustomMenuItems(menu) {
  menu.AddItem(refreshMenu, 0, RefreshDisplay);
}

function CheckForUpdates() {
}

function PostRefreshTasks() {
//  CheckForUpdates();
}

function RefreshDisplay() {
  var url = null;
  url = kHatenaUsernameTemplate.replace(/\{user\}/g,
                                        options.GetValue("Username"));
  http = new XMLHttpRequest();
  http.onreadystatechange = OnReceivedData;
  // http.setRequestHeader("If-Modified-Since", "Sat, 1 Jan 2000 00:00:00 GMT");
//  http.open("GET", url + "?noCache=" + Math.random(), true);
  http.open("GET", url, true);
  try {
    http.send();
  }
  catch(e) {
    // Just in case ... not that we expect something bad to happen here.
  }
}

function OnReceivedData() {
  if (http && http.readyState == 4) { // completed OK?
    // make array empty.
    for (var i = 0; i < contentItems.length; ++i) {
      contentItems[i] = null; // dereference each element.
    }
    contentItems.splice(0, contentItems.length);

    // Parse and add each antenna to our array.
//    utils.alert(http.responseText);
    var doc = new DOMDocument();
    doc.loadXML(http.responseText);

    var currentDate = null;
    if(kShowsFirstDate == 1){
      currentDate = 1;
    }
    var title, url, desc, date;
    var day = 0;
    var firstAttempt = false;
    var firstDate;
    var timeOffset;
    var elem = doc.getElementsByTagName("item");
    if (elem != null && elem.length > 0) {
      for(var i = 0; i < elem.length; ++i) {
        for (var node = elem[i].firstChild; node != null;
            node = node.nextSibling) {
          if (node.nodeName == "title")
            title = node.firstChild.nodeValue;
          else if (node.nodeName == "link")
            url = node.firstChild.nodeValue;
          else if (node.nodeName == "description"){
            if( (kDetail == 1) && (node.firstChild != null) ){
              desc = node.firstChild.nodeValue;
            }
            else{
              desc = "";
            }
          }
          else if (node.nodeName == "dc:date"){
            date = node.firstChild.nodeValue;
            date = date.replace(/T/, ' ');
            timeOffset = date.slice( date.indexOf("+") );
            date = date.slice( 0, date.indexOf("+") );
            day = date.slice( date.indexOf("-")+1, date.indexOf(" ") );
          }
        }

        if (title != null) {  // successfully parsed?
          if(firstAttempt == false){
            firstDate = parseTime(date, timeOffset);
            firstAttempt = true;
          }
          if((day != currentDate) && (kShowsDate == 1)){
            if(currentDate != null){
              var dayItem = new AntennaItem(day, "","","", false, null);
              contentItems.push(dayItem);
            }
            currentDate = day;
          }
          var theUpdatedDate = parseTime(date, timeOffset);
          var antennaItem = new AntennaItem(title, url, desc, date, IsNew( theUpdatedDate ), theUpdatedDate );
          contentItems.push(antennaItem);
        }
      }
    }

    // http = null;

    antenna.removeAllContentItems();
    contentItems = contentItems.reverse();
    
//    utils.alert("Adding : " + contentItems.length + " items.");
    for (var i = 0; i < contentItems.length; ++i) {
      contentItems[i].AddToSidebar();
    }

    options.PutValue("Lastupdate", firstDate.toString());
    kLastUpdate = firstDate.getTime();
//    utils.alert( "Lastly = " + options.GetValue("Lastupdate") );
    PostRefreshTasks();
  }
}

function parseTime( date, to ){
  var dateAndTime = date.split(" ");
  var theDateArr = dateAndTime[0].split("-");
  var theTimeArr = dateAndTime[1].split(":");
  
  var theYear = theDateArr[0];
  var theMonth = theDateArr[1];
  var theDate = theDateArr[2];
  var theHour = theTimeArr[0];
  var theMin = theTimeArr[1];
  var theSec = theTimeArr[2];
  
  var thisDate = new Date(theYear, theMonth-1, theDate, theHour, theMin, theSec);
  var localeDate = new Date();
  thisDate.setTime( thisDate.getTime() - ( (parseTimeOffset( to ) * 60) * 60 * 1000 ) );

//  utils.alert(localeDate.getTimezoneOffset() / 60);
//  utils.alert(thisDate.toString());
  return thisDate;

}

function parseTimeOffset( to ){
  var ret = 0;
  ret = parseInt( to.slice( 0, to.indexOf(":") ), 10 );
  
  if(to.charAt(0) == "-"){
    ret = ret * -1;
  }
//  utils.alert(ret);
  return ret; // in hour
}


function IsNew( date ){

  if( date.getTime() > kLastUpdate ){
    return true;
  }
  else return false;
}


// A class that encapsulates a antenna.
function AntennaItem(title, url, desc, date, inew, theDate) {
  this.item = new ContentItem();

//  this.item.layout = gddContentItemLayoutNowrapItems; // gddContentItemLayoutEmail;
  this.item.layout = gddContentItemLayoutNews; // gddContentItemLayoutEmail;
  this.item.heading = title;
  this.item.source = url;
//  utils.alert(theDate.getVarDate());
  if(theDate != null){
    var tmD = new Date();
    if( (tmD.getTime() - theDate.getTime()) <= 1000 * 60 * 60 * 24 * 3 ){
      this.item.time_created = theDate.getVarDate();
    }
  }
//  this.item.tooltip = url + "\n" + desc;
  this.fnew = inew;
  if(url == ""){
    this.item.tooltip = date;
    this.item.flags = gddContentItemFlagHighlighted;
  }
  else {
    this.item.tooltip = url + "\n" + date + "\n" + desc;
  }
//  this.item.snippet = options.GetValue("url:" + url);
  this.item.snippet = desc;

  this.item.onDetailsView = OnOpenItem; // Open item even on single-click.
  this.item.onOpenItem = OnOpenItem; // And keep double-click behavior unchanged.

  // Methods.


  AntennaItem.prototype.AddToSidebar = AddToSidebar;
  function AddToSidebar() {
//     utils.alert(this.item.source + "\n" + options.GetValue("url:" + this.item.source));
    if( this.fnew == true ){
      antenna.addContentItem(this.item, gddItemDisplayInSidebar | gddItemDisplayAsNotification);
    }
    else{
      antenna.AddContentItem(this.item, gddItemDisplayInSidebar);
    }
  }


  function OnOpenItem(item) {

    if(item.source != ""){
      // Since we take over the default behavior, we need to open the item ourselves.
      var wshShell = new ActiveXObject("WScript.Shell");
      wshShell.run(item.source);
      wshShell = null;
    }
  }

}

function GetProperty(url, index) {
  var currValue = options.GetValue(url);
  if (currValue != "") {
    var props = currValue.split(",");
    return props[index];
  } else {
    return "";
  }
}

function PutProperty(url, index, newValue) {
  // assert (index < kPropertyCount);
  var oldValue = options.GetValue(url);
  var oldProps;
  if (oldValue != "") {
    oldProps = oldValue.split(",");
  } else {
    oldProps = new Array(kPropertyCount); // make new properties if missing.
  }
  oldProps[index] = newValue;
  options.PutValue(url, oldProps.join(","));
}

// ---- Options Dialog ----

function ShowOptionsDlg(wnd) {
  wnd.AddControl(gddWndCtrlClassLabel, 0, "", enterHatenaUsername,
      30, 10, 200, 15);
  wnd.AddControl(gddWndCtrlClassEdit, 0, "username", options.GetValue("Username"),
      30, 25, 150, 22);
  var defaultButton = wnd.AddControl(gddWndCtrlClassButton, 0,
      "resetUserIDButton", resetUserIDButton, 230, 25, 100, 22);

  wnd.AddControl(gddWndCtrlClassLabel, 0, "", enterTimeInterval,
                 30, 50, 200, 15);
  wnd.AddControl(gddWndCtrlClassEdit, 0, "timeinterval", options.GetValue("Timeinterval"),
                 30, 65, 150, 22);

  var checkBox = wnd.AddControl(gddWndCtrlClassButton, gddWndCtrlTypeButtonCheck, "showsdate", "",
                 30, 100, 10, 10);
  if( options.GetValue("Showsdate") == 1 ){ checkBox.value = true; }
  else { checkBox.value = false; }
  wnd.AddControl(gddWndCtrlClassLabel, 0, "", enterShowsDate,
                 50, 100, 170, 30);

  var checkBox2 = wnd.AddControl(gddWndCtrlClassButton, gddWndCtrlTypeButtonCheck, "showsfirstdate", "",
                 30, 130, 10, 10);
  if( options.GetValue("Showsfirstdate") == 1 ){ checkBox2.value = true; }
  else { checkBox2.value = false; }
  wnd.AddControl(gddWndCtrlClassLabel, 0, "", enterShowsFirstDate,
                 50, 130, 170, 30);

  var checkBox3 = wnd.AddControl(gddWndCtrlClassButton, gddWndCtrlTypeButtonCheck, "showsdetail", "",
                 30, 160, 10, 10);
  if( options.GetValue("Showsdetail") == 1 ){ checkBox3.value = true; }
  else { checkBox3.value = false; }
  wnd.AddControl(gddWndCtrlClassLabel, 0, "", enterShowsDetail,
                 50, 160, 170, 30);

  
  defaultButton.onClicked = OnResetUserIDButtonClicked;

  wnd.onClose = OptionsDlgClosed;
}


function OnResetUserIDButtonClicked(wnd, ctrl){
  // Reset the userID (change the ID to default)
  var currentHID = wnd.GetControl("username");
  currentHID.value = kDefaultUsername;
}

function OptionsDlgClosed(wnd, code) {
  if (code == gddIdOK) {
    var showsdate = wnd.GetControl("showsdate");
    if( showsdate.value == true ){
      kShowsDate = 1;
      options.PutValue("Showsdate", 1);
    }
    else{
      kShowsDate = 0;
      options.PutValue("Showsdate", 0);
    }


    var showsfirstdate = wnd.GetControl("showsfirstdate");
    if( showsfirstdate.value == true ){
      kShowsFirstDate = 1;
      options.PutValue("Showsfirstdate", 1);
    }
    else{
      kShowsFirstDate = 0;
      options.PutValue("Showsfirstdate", 0);
    }

    var showsdetail = wnd.GetControl("showsdetail");
    if( showsdetail.value == true ){
      kDetail = 1;
      options.PutValue("Showsdetail", 1);
    }
    else{
      kDetail = 0;
      options.PutValue("Showsdetail", 0);
    }
    
    var username = wnd.GetControl("username");
    options.PutValue("Username", username.text);
    var timeinterval = wnd.GetControl("timeinterval");
    options.PutValue("Timeinterval", timeinterval.text);
    ResetTimeInterval(timeinterval.text);
    

    RefreshDisplay();
  }
}
